"""
Provides routes related to managing ConsumptionType objects.
"""

from sqlalchemy.exc import SQLAlchemyError
from flask import jsonify, request

from piket_server.models import ConsumptionType
from piket_server.flask import app, db


@app.route("/consumption_types", methods=["GET"])
def get_consumption_types():
    """ Return a list of currently active consumption types. """
    try:
        active = int(request.args.get("active", 1))

    except ValueError:
        return {}, 400

    ctypes = ConsumptionType.query.filter_by(active=active).all()
    result = [ct.as_dict for ct in ctypes]
    return jsonify(consumption_types=result)


@app.route("/consumption_types/<int:consumption_type_id>", methods=["GET"])
def get_consumption_type(consumption_type_id: int):
    ct = ConsumptionType.query.get_or_404(consumption_type_id)

    return jsonify(consumption_type=ct.as_dict)


@app.route("/consumption_types", methods=["POST"])
def add_consumption_type():
    """ Add a new ConsumptionType.  """
    json = request.get_json()

    if not json:
        return jsonify({"error": "Could not parse JSON."}), 400

    data = json.get("consumption_type") or {}
    ct = ConsumptionType(name=data.get("name"), icon=data.get("icon"))

    try:
        db.session.add(ct)
        db.session.commit()
    except SQLAlchemyError:
        return jsonify({"error": "Invalid arguments for ConsumptionType."}), 400

    return jsonify(consumption_type=ct.as_dict), 201


@app.route("/consumption_types/<int:consumption_type_id>", methods=["PATCH"])
def activate_consumption_type(consumption_type_id: int):
    ct = ConsumptionType.query.get_or_404(consumption_type_id)

    data = request.json["consumption_type"]
    new_active = data.get("active", True)

    ct.active = new_active
    db.session.add(ct)
    db.session.commit()

    return jsonify(consumption_type=ct.as_dict), 200
